/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "SimulatorThread.h"
#include "ThyrixApplication.h"
#include "ThyrixMainFrame.h"
#include "World.h"

SimulatorThread::SimulatorThread(ThyrixMainFrame *iniFrame) :
	wxThread(wxTHREAD_JOINABLE),
	frame(iniFrame),
	timeFactor(1),
	paused(false),
	expectedTime(0){ 			
	setTimeFactor(1);
	setFramesPerSecond(30);
}

SimulatorThread::~SimulatorThread(){
}

void SimulatorThread::step() {
		//this is called directly by the parent frame, following a command
    //needs mutex sincronization with the thread.
    world->advanceTime();
}

void SimulatorThread::setPause() {
    paused = true;
}

void SimulatorThread::setTimeFactor(float newFactor) {
    timeFactor = newFactor;
    paused = false;
    expectedTime = 0;
    simulationTimer.Start();
}

void SimulatorThread::setFramesPerSecond(int fps) {
    frameInterval = 1000 / fps;
}

void SimulatorThread::sleepIfAhead() {
	
    //sleep when more than 5ms ahead
    static const int kSleepThreshold = 5;

    unsigned expectedTimeMs = (unsigned)(expectedTime * 1000);
    unsigned realMs = (unsigned)simulationTimer.Time();
    int aheadMs = 0;
    if ( expectedTimeMs > realMs) {
        aheadMs = expectedTimeMs - realMs;
    }
    if (aheadMs > kSleepThreshold) {
        Sleep(aheadMs);
    }
    //to avoid timer overflow, reset it once in a while
    if (realMs > 100000) {
        setTimeFactor(timeFactor);
    }
}

void* SimulatorThread::Entry(){
	wxStopWatch displayTimer;
	simulationTimer.Start(); //starts the timing of the running time of the simulator
	while(!TestDestroy()){
		displayTimer.Start(); //starts the timing of the duration of a frame   
		frame->paint(); //draws the image
		if (paused) {
			Sleep(frameInterval);
		} else {

			//advance simulator time, in a loop, as long a refresh of the display is not needed
			do {
				
				//wait if the simulation should not advance yet
				sleepIfAhead();

				//advance simulator time
				world->advanceTime();
				
				//computes the real time at which the next simulator timestep should be executed,
				//given the desired acceleration factor between real time and simulator time
				expectedTime += world->getDt() / timeFactor;

			} while (displayTimer.Time() < frameInterval);

		}
	}
	return NULL;
}

void SimulatorThread::OnExit(){
}
