/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#ifndef THYRIXMAINFRAME_H
#define THYRIXMAINFRAME_H

#include "SimulatorThread.h"
#include "GUIWx.h"

class World;

// Define a new frame type: this is going to be our main frame
class ThyrixMainFrame : public wxFrame{
public:
    enum {Unused = 100,
          kMenuTimePause,
          kMenuTimeDiv10,
          kMenuTimeX1,
          kMenuTimeX3,
          kMenuTimeX10,
          kMenuTimeX30,
          kMenuTimeX100,
          kMenuTimeMax,
          
          kMenuFps30,
          kMenuFps60,
          kMenuFps90,
          
          kMenuStep,
					kMenuCapture
    };

    

	// constructors, destructors
	ThyrixMainFrame(const wxString& title, World* world, 
		int bufferWidth=600, int bufferHeight=400, int zoom=100);
	virtual ~ThyrixMainFrame();

	/** Creates the GUI object (needed by the simulator) and starts the thread. */
	void start();

	void buildMenu();

	// event handlers (these functions should _not_ be virtual)
	void OnClose(wxCloseEvent& event);
	//void OnQuit(wxCommandEvent& event);
	//void OnAbout(wxCommandEvent& event);
	
	void onSetSpeed(wxCommandEvent& event);
	void onSetFps(wxCommandEvent& event);
	

	//void OnIdle(wxIdleEvent& event);

	/** Paints the image to the screen. This is used on Paint events. */
	//void onPaint(wxPaintEvent& event);

	/** Paints the image to the screen. This is used on events issued by the simulator thread. */
	//void onThreadPaintCommand(wxCommandEvent& event);
	
	/** Not used now. Use this to customize background erasing. */
	//void onEraseBackground(wxEraseEvent& event);
	
	/** Captures mouse motion. */
	void onMotion(wxMouseEvent& event);

	/** Captures pressing of mouse left button. */
	void onLeftDown(wxMouseEvent& event);

	/** Captures release of mouse left button. */
	void onLeftUp(wxMouseEvent& event);

	void onRightDown(wxMouseEvent& event);
	void onRightUp(wxMouseEvent& event);
	
	void onStep(wxCommandEvent& event);
	void onCapture(wxCommandEvent& event);
	
	/** Paints the simulator generated image to the specified device context. */
	void paintDC(wxDC* dc);
	
	/** Paints the image to the memory buffer and then to the screen. */
	void paint();

	World* world;

private:
	//Used inside a class declaration to declare that the class should be 
	//made known to the class hierarchy, but objects of this class cannot 
	//be created dynamically
	// (?)
	DECLARE_CLASS(ThyrixMainFrame); //?

	// any class wishing to process wxWindows events must use this macro
	DECLARE_EVENT_TABLE();

	SimulatorThread thread;
	
	/** Memory device context used as a graphical buffer. It provides a means to draw on a bitmap. */
	wxMemoryDC buffer;
	
	/** Bitmap used for graphical buffering. */
	wxBitmap bitmap;
	
	/** The dimensions of the graphical buffer. */
	int bufferWidth, bufferHeight;

	/** The GUI object (an interface to the GUI which is available to the simulator). */
	GUIWx *gui;

	/** The zoom applied by the GUI object. */
	int zoom;
};

#endif // THYRIXMAINFRAME_H
