/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Pac.cpp: implementation of the Pac class.
//
//////////////////////////////////////////////////////////////////////

#include "Pac.h"
#include "Circle.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Pac::Pac(real x0, real y0) : controller(0,5){
 int i;
 nCircles=11;
 rPac=0.4;
 u=2*M_PI/nCircles;
 u0=u/2;
 Circle* c;
 for(i=0;i<nCircles;i++){
	c=new Circle(0.1,x0+rPac*cos(u0+i*u),y0+rPac*sin(u0+i*u));
	c->setColor(Color("slateGray"));
	addObject(c);
 }
 real k=3;
 for(i=0;i<nCircles;i++){
	links.push_back(ElasticLink(objects[index(i-1)],objects[i],2*rPac*sin(u/2),k));
	links.push_back(ElasticLink(objects[index(i-2)],objects[i],2*rPac*sin(u),k));
	links.push_back(ElasticLink(objects[index(i-3)],objects[i],2*rPac*sin(3*u/2),k));
	links.push_back(ElasticLink(objects[index(i-4)],objects[i],2*rPac*sin(2*u),k));
 }
 activeLink=&(*(links.begin()));
 activeLink->k=4*k;
 l=2*rPac*sin(u/2);
 label="Pac";
}

Pac::~Pac(){

}

void Pac::controll(){
	activeLink->length=l*(1+2*controller.getOutput(0));
	real thrust=controller.getOutput(1)*0.5;
	Vector2 n=objects[0]->r;
	n-=objects[nCircles-1]->r;
	n.normalize();
	n.rotate(-M_PI/2);
	Vector2 force=n;
	force*=thrust;
	PhysicalObjectPVector::iterator it, end=objects.end();
	for(it=objects.begin(); it!=end; ++it){
		(*it)->externalForce+=force;
	}
	real rotation=(controller.getOutput(2)-controller.getOutput(3))*0.5;
	n*=rotation;
	n.rotate(M_PI/2+u0);
	for(it=objects.begin(); it!=end; ++it){
		(*it)->externalForce+=n;
		n.rotate(u);
	}
	

	controller.advanceTime();
}