/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Spherus.h: interface for the Spherus class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SPHERUS_H
#define SPHERUS_H


#include "PhysicalObject.h"
#include "VisualSensor.h"


class Controller;
class Circle;

/** Defines the Spherus agent. The agent is composed of two circles, each at distance l
from the center r of the agent. The angle of the line defined by the circles with the 
horisontal is alpha.
*/


class Spherus : public PhysicalObject  {
public:
	Spherus(real x, real y, real alpha=0.0, real extension=0.6, real R=0.1, Controller* controller=NULL);
	virtual ~Spherus();

	
	/** The radius of the circles composing the agent */
	real R;
	
	/** The distance between the circles. */
	real extension;	

	/** The circles composing the agent */
	Circle* circles[2];

	VisualSensor* eyes[2];

	real rocketActivations[2];
	real rocketForceValues[2];
	Vector2 rocketForces[2];
	real extensionActivation;

	/** The target distance between the circles*/
	real targetExtension;

	/** Computes alpha and extension as a function of the positions of the 
	two eyes, calls computeSinCos. */
	void computeState();

	real sinAlpha, cosAlpha;

	void computeSinCos();

	void computeMemberPositions();

	void computeCircleProprioception(Circle* circle, float& forward, float& backward);

	virtual void computeDerivativesWithoutContacts(ContactSolver* contactSolver);
	virtual void computeDerivatives(GlobalContactInfoVector* globalContacts);
	virtual void integrate(const Integrator &integrator);
	virtual void controll();

	virtual bool detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts);
	virtual void detectInternalContacts(GlobalContactInfoVector* contacts);
	virtual void deleteContacts();
	virtual bool detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object);


	void draw(GUI *gui);

	static const unsigned int nSensors;
	static const unsigned int nEffectors;

	Controller* controller;

};

#endif //SPHERUS_H
