/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "ContactInfo.h" 
#include "PhysicalObject.h"


ContactInfo::ContactInfo(PhysicalObject* iniObject, int iniSigma) :
	sigma(iniSigma),
	type(contactTypeTorque),
	object(iniObject),
	force(0.0){
		parentObject=object;
}

ContactInfo::ContactInfo(PhysicalObject* iniObject, int iniSigma, 
                         const Vector2& iniP, const Vector2& iniN, int iSensor) :
	object(iniObject),    
	sigma(iniSigma),
	p(iniP),
	n(iniN),
	iSensor(iSensor),
	type(contactTypeForce),
	force(0.0){ 
	setupObject();
}


ContactInfo::ContactInfo(PhysicalObject* iniObject, int iniSigma, 
                         const Vector2& iniP, const Vector2& iniN, 
						 int iSensor, int iSensor2) :
	object(iniObject),
	sigma(iniSigma),
	p(iniP),
	n(iniN),
	iSensor(iSensor),
	iSensor2(iSensor2),
	type(contactTypeForceParallel),
	force(0.0){ 	
	setupObject();	
}

void ContactInfo::setupObject(){
	if(object->parent){
		/*for composed objects, modify the relative position, 
          to be relative to the origin of the
          composed object, rather than the object itself */
		Vector2	rt=object->r;
		rt -= object->parent->r;
		p += rt;
		/*for composed objects, the contact is registered 
          at the level of the composed object, 
          not at the level of the simple objects */
		parentObject = object->parent;
	} else {
		parentObject = object;
	}
	pxn = p ^ n;
	parentObject->addContact(this);
}

ContactInfo::~ContactInfo() {
}
