/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// Elastoid.cpp: implementation of the Elastoid class.
//
//////////////////////////////////////////////////////////////////////

#include "Elastoid.h"
#include "Simulator.h"
#include "purgeContainer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Elastoid::Elastoid(){
}

Elastoid::~Elastoid(){
	purgeContainer(objects);
}

void Elastoid::registerPrimitives(Simulator* simulator){
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		simulator->registerPrimitive(*it);
  }
}

void Elastoid::computeDerivativesWithoutContacts(ContactSolver* contactSolver){
	for(ElasticLinkVector::iterator lit=links.begin(), lend=links.end(); lit!=lend; ++lit)
		(*lit).applyForces();
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
																		end=objects.end(); it!=end; ++it)
		(*it)->computeDerivativesWithoutContacts(contactSolver);
}

void Elastoid::computeDerivatives(GlobalContactInfoVector* globalContacts){
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
																		end=objects.end(); it!=end; ++it)
		(*it)->computeDerivatives(globalContacts);
}

void Elastoid::integrate(const Integrator &integrator){
	boxMin.setXY(ThyrixParameters::infinity,ThyrixParameters::infinity);
	boxMax.setXY(-ThyrixParameters::infinity,-ThyrixParameters::infinity);
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
		end=objects.end(); it!=end; ++it){
		(*it)->integrate(integrator);
		boxMin.updateMin((*it)->boxMin);
		boxMax.updateMax((*it)->boxMax);
	}
}

bool Elastoid::detectContacts(PhysicalObject* object, GlobalContactInfoVector* contacts) {
	bool isContact=false;
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
																		end=objects.end(); it!=end; ++it)
		isContact = (*it)->detectContacts(object, contacts) || isContact;
	return isContact;
}

void Elastoid::detectInternalContacts(GlobalContactInfoVector* globalContacts) {
	PhysicalObjectPVector::iterator it1, it2, end=objects.end();
	for(it1=objects.begin(); it1!=end; ++it1)
		for(it2=it1, ++it2; it2!=end; ++it2)
			(*it1)->detectContacts(*it2, globalContacts);
}

void Elastoid::deleteContacts(){ 
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
																		end=objects.end(); it!=end; ++it)
		(*it)->deleteContacts();
}

bool Elastoid::detectMouseContact(const Vector2& rMouse, Vector2& p, PhysicalObject*& object){
	bool isContact=false;
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
			end=objects.end(); it!=end && !isContact; ++it){
			isContact = (*it)->detectMouseContact(rMouse, p, object);
		}
	return isContact;
}



void Elastoid::draw(GUI* gui){
	gui->setPenColor(Color("lightGray"));
	for(ElasticLinkVector::iterator lit=links.begin(), lend=links.end(); lit!=lend; ++lit)
		(*lit).draw(gui);
	for(PhysicalObjectPVector::iterator it=objects.begin(), 
            end=objects.end(); it!=end; ++it) {
		(*it)->draw(gui);
	}
}