/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "SystemSolver.h"
#include "MathTools.h"
#include <stdlib.h>
#include <assert.h>

#define TINY 1.0e-20;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SystemSolver::SystemSolver(){
}

SystemSolver::~SystemSolver(){
}

void SystemSolver::luDecompose(real** a, int n, int* index, real* vv){
	int i,imax,j,k;
	real big,dum,sum,temp;

	//vv=(real*)malloc(n * sizeof(real));
	for (i=0;i<n;i++) {
		big=0.0;
		for (j=0;j<n;j++)
			if ((temp=fabs(a[i][j])) > big) big=temp;
		assert (big != 0.0);
		vv[i]=1.0/big;
	}
	for (j=0;j<n;j++) {
		for (i=0;i<j;i++) {
			sum=a[i][j];
			for (k=0;k<i;k++) sum -= a[i][k]*a[k][j];
			a[i][j]=sum;
		}
		big=0.0;
		for (i=j;i<n;i++) {
			sum=a[i][j];
			for (k=0;k<j;k++)
				sum -= a[i][k]*a[k][j];
			a[i][j]=sum;
			if ( (dum=vv[i]*fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if (j != imax) {
			for (k=0;k<n;k++) {
				dum=a[imax][k];
				a[imax][k]=a[j][k];
				a[j][k]=dum;
			}
			//*d = -(*d);
			vv[imax]=vv[j];
		}
		index[j]=imax;
		if (a[j][j] == 0.0f) a[j][j]=TINY;
		if (j != n-1) {
			dum=1.0/(a[j][j]);
			for (i=j+1;i<n;i++) a[i][j] *= dum;
		}
	}
	//free(vv);
}

void SystemSolver::luSubstitute(real** a, int n, int* index, real* b){
	int i,ii=-1,ip,j;
	real sum;

	for (i=0;i<n;i++) {
		ip=index[i];
		sum=b[ip];
		b[ip]=b[i];
		if (ii>=0)
			for (j=ii;j<=i-1;j++) sum -= a[i][j]*b[j];
		else if (sum) ii=i;
		b[i]=sum;
	}
	for (i=n-1;i>=0;i--) {
		sum=b[i];
		for (j=i+1;j<n;j++) sum -= a[i][j]*b[j];
		b[i]=sum/a[i][i];
	}
}

void SystemSolver::svDecompose(real** a, int n, real* w, real** v, real* rv1){
	int flag,i,its,j,jj,k,L,nm;
	real anorm,c,f,g,h,s,scale,x,y,z;

	g=scale=anorm=0.0;
	for (i=0;i<n;i++) {
		L=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;

		for (k=i;k<n;k++) scale += fabs(a[k][i]);
		if (scale) {
			for (k=i;k<n;k++) {
				a[k][i] /= scale;
				s += a[k][i]*a[k][i];
			}
			f=a[i][i];
			g = -MathTools::setSign(sqrt(s),f);
			h=f*g-s;
			a[i][i]=f-g;
			for (j=L;j<n;j++) {
				for (s=0.0,k=i;k<n;k++) s += a[k][i]*a[k][j];
				f=s/h;
				for (k=i;k<n;k++) a[k][j] += f*a[k][i];
			}
			for (k=i;k<n;k++) a[k][i] *= scale;
		}
		
		w[i]=scale *g;
		g=s=scale=0.0;
		if (i < n-1) {
			for (k=L;k<n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=L;k<n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][L];
				g = -MathTools::setSign(sqrt(s),f);
				h=f*g-s;
				a[i][L]=f-g;
				for (k=L;k<n;k++) rv1[k]=a[i][k]/h;
				for (j=L;j<n;j++) {
					for (s=0.0,k=L;k<n;k++) s += a[j][k]*a[i][k];
					for (k=L;k<n;k++) a[j][k] += s*rv1[k];
				}
				for (k=L;k<n;k++) a[i][k] *= scale;
			}
		}
		anorm=MathTools::max(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
	for (i=n-1;i>=0;i--) {
		if (i < n-1) {
			if (g) {
				for (j=L;j<n;j++)
					v[j][i]=(a[i][j]/a[i][L])/g;
				for (j=L;j<n;j++) {
					for (s=0.0,k=L;k<n;k++) s += a[i][k]*v[k][j];
					for (k=L;k<n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=L;j<n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		L=i;
	}
	for (i=n-1;i>=0;i--) {
		L=i+1;
		g=w[i];
		for (j=L;j<n;j++) a[i][j]=0.0;
		if (g) {
			g=1.0/g;
			for (j=L;j<n;j++) {
				for (s=0.0,k=L;k<n;k++) s += a[k][i]*a[k][j];
				f=(s/a[i][i])*g;
				for (k=i;k<n;k++) a[k][j] += f*a[k][i];
			}
			for (j=i;j<n;j++) a[j][i] *= g;
		} else for (j=i;j<n;j++) a[j][i]=0.0;
		++a[i][i];
	}
	for (k=n-1;k>=0;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (L=k;L>=0;L--) {
				nm=L-1;
				if ((float)(fabs(rv1[L])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((float)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=L;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((float)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=MathTools::modulus(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s = -f*h;
					for (j=0;j<n;j++) {
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (L == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=0;j<n;j++) v[j][k] = -v[j][k];
				}
				break;
			}
			if (its == 30) exit(0); //no convergence in 30 svdcmp iterations
			x=w[L];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=MathTools::modulus(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+MathTools::setSign(g,f)))-h))/x;
			c=s=1.0;
			for (j=L;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=MathTools::modulus(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g = g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=0;jj<n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=MathTools::modulus(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=c*g+s*y;
				x=c*y-s*g;
				for (jj=0;jj<n;jj++) {
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			rv1[L]=0.0;
			rv1[k]=f;
			w[k]=x;
		}
	}
}


void SystemSolver::svSubstitute(real** u, int n, real* w, real** v, real* b, real* t){
	real wmax=-1e10, wmin, s;
	int i,j,k;
	for(i=0;i<n;i++) 
		if(w[i]>wmax) wmax=w[i];
	wmin=wmax*1e-6;
	for(i=0;i<n;i++) 
		if(w[i]<wmin) w[i]=0;

	for (j=0;j<n;j++) {
		s=0.0;
		if (w[j]) {
			for (i=0;i<n;i++) s += u[i][j]*b[i];
			s /= w[j];
		}
		t[j]=s;
	}
	for (j=0;j<n;j++) {
		s=0.0;
		for (k=0;k<n;k++) s += v[j][k]*t[k];
		w[j]=s;
	}
}