/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ThyrixParameters.h: interface for the ThyrixParameters class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_THYRIXPARAMETERS_H__442475F3_9963_4247_8192_1505B7F7E4D5__INCLUDED_)
#define AFX_THYRIXPARAMETERS_H__442475F3_9963_4247_8192_1505B7F7E4D5__INCLUDED_

#include <math.h>

/** The basic floating point type used by the simulator for representing physical quantities.*/
typedef double real;

/** Square function. */
template<class Number> inline Number sqr(Number x){ return x*x; }

/** Pi constant.*/
#ifndef M_PI
	#define M_PI 3.1415926535897932384626433832795028841971693993751058209749445923078164062862
#endif

/** A degree in radians. */
const real degrees=M_PI/180.0;


//#define ARRAYSIZE(x) (sizeof(x) / sizeof(x[0]))

/** Defines some basic parameters for the simulator. */

class ThyrixParameters {
public:
	ThyrixParameters();
	virtual ~ThyrixParameters();

	/** Tolerance length for estimating contact at contact detection. */
	static const real epsilonContact;

	/** 1/2 epsilonContact. */
	static const real halfEpsilonContact;

	/** The default surface density of objects in the simulator. */
	static const real defaultDensity;

	/** The elastic constant used for penalty forces in case of penetration. */
	static const real kContact;

	/** A number larger than the values of any coordinates of objects in the simulator. */
	static const real infinity;

	/** An elastic constant used for computing the force generated by dragging an object with the mouse. */
	static real kMouseForce;

};

#endif // !defined(AFX_THYRIXPARAMETERS_H__442475F3_9963_4247_8192_1505B7F7E4D5__INCLUDED_)
