/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// World.h: interface for the World class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WORLD_H__A1A33537_BF98_497E_B4CF_3A2839E9C233__INCLUDED_)
#define AFX_WORLD_H__A1A33537_BF98_497E_B4CF_3A2839E9C233__INCLUDED_

#include "GUI.h"

const real dtDefault=(real)0.02;

/** A world is a dynamic system that is updated in discrete time and can be graphically illustrated. 
This class provides just an interface, and implements interaction through mouse events. */
class World  {
public:
	World();
	virtual ~World();

	virtual double getDt() const { return (double) dtDefault; }

	virtual void advanceTime()=0;

	virtual void draw(GUI *gui);

	/** Coordinates, in world space, of the position of the mouse. */
	float mouseX, mouseY;
	/** This may be used for sending custom events.*/
	bool isRightMouseButtonPressed;
	bool isLeftMouseButtonPressed;

	virtual void setMouseCoordinates(float x, float y){
		mouseX=x;
		mouseY=y;
	}

	virtual void onMouseLeftDown(float x, float y){
		setMouseCoordinates(x,y);
		isLeftMouseButtonPressed=true;
	}

	virtual void onMouseRightDown(float x, float y){
		setMouseCoordinates(x,y);
		isRightMouseButtonPressed=true;
	}

	virtual void onMouseLeftUp(float x, float y){
		setMouseCoordinates(x,y);
		isLeftMouseButtonPressed=false;
	}

	virtual void onMouseRightUp(float x, float y){
		setMouseCoordinates(x,y);
		isLeftMouseButtonPressed=false;
	}
	
	unsigned long int timeStep;

};

#endif // !defined(AFX_WORLD_H__A1A33537_BF98_497E_B4CF_3A2839E9C233__INCLUDED_)
