/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ArticulatedAgentQuasistatic.h: interface for the ArticulatedAgentQuasistatic class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ARTICULATEDAGENTQUASISTATIC_H
#define ARTICULATEDAGENTQUASISTATIC_H

#include "ArticulatedAgentBase.h"


/** Computes the velocities of the coordinates for an articulated agent, given forces,
using a Featherstone-type algorithm, for the case of quasistatic mechanics. */

class ArticulatedAgentQuasistatic : public ArticulatedAgentBase  {

public:

	/** Default constructor; sets the simulator. */
	ArticulatedAgentQuasistatic(std::string label="");
	/** Destructor; does nothing. */
	virtual ~ArticulatedAgentQuasistatic();

protected:
	/** Based on the position of the body and the thetas, computes the position and angle of each of the 
	links.*/
	void forwardKinematics();
	/** Implements the Backward Dynamics step of the Featherstone algorithm. */
	void backwardDynamics();
	/** Implements the Forward Accelerations step of the Featherstone algorithm. */
	void forwardAccelerations(ContactSolver* contactSolver);
	
	/** Computes the derivatives through the Featherstone algorithm. */
	virtual void computeDerivativesWithoutContacts(ContactSolver* contactSolver){
		backwardDynamics();
		forwardAccelerations(contactSolver);
	}

	/** Computes the body derivatives without contacts. It is externalized in order to allow 
	the possibility of a fixed body. */
	virtual void computeBodyDerivativesWithoutContacts(real determinant);

	virtual void computeDerivatives(GlobalContactInfoVector* globalContacts);

	/** Computes the body derivatives. It is externalized in order to allow the possibility of a 
	fixed body. */
	virtual void computeBodyDerivatives(GlobalContactInfoVector* globalContacts);

	void computeForces(GlobalContactInfoVector* globalContacts);

	/** Computes the total forces and torques acting on a link; used for testing purposes. */
	void computeTotalForces(GlobalContactInfoVector* globalContacts);



	/** Advances the time to the next timestep, updating the positions. */
	virtual void integrate(const Integrator &integrator);

	virtual void rollback();
};

#endif //ARTICULATEDAGENTCLASSICAL_H
