/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

#include "ArticulatedLimb.h"
#include "Simulator.h"
#include "purgeContainer.h"

/*
ArticulatedLimb::ArticulatedLimb(){
	l=0.0;
	theta=0.0;
}
*/

ArticulatedLimb::ArticulatedLimb(real iniL, real iniTheta) :
    l(iniL),
    theta(iniTheta)
{
}

ArticulatedLimb::~ArticulatedLimb(){
}


ArticulatedLink* ArticulatedLimb::addLink() {
	int n = links.size() + 1;
	char s[20];
	sprintf(s, "%d", n);
	ArticulatedLink* link=new ArticulatedLink(label + ": link " + s);
	links.push_back(link);
	return link;
}

ArticulatedLink* ArticulatedLimb::addOneObjectLink(real l, real theta0, 
																	PhysicalObject* object, 
																	real x, real y, real alpha,
																	ArticulatedLink* parent, bool hasElasticTorque){
	ArticulatedLink* link;
	link=addLink();
	link->l=l;
	link->theta0=theta0;
	link->theta=link->theta0;
	link->computeSinCos();
	link->addObject(object, x, y, alpha);
	link->computeIStar0();
	link->setParentLink(parent);
	link->hasElasticTorque=hasElasticTorque;
	return link;
}

void ArticulatedLimb::computeSinCos(){
	sinTheta=sin(theta);
	cosTheta=cos(theta);
	sin2=sinTheta*sinTheta;
	cos2=cosTheta*cosTheta;
	sinCos=sinTheta*cosTheta;
}

void ArticulatedLimb::setProperties(real l, real theta) {
	this->l=l;
	this->theta=theta;
	computeSinCos();
}

void ArticulatedLimb::deleteLinks(){
	purgeContainer(links);
}

void ArticulatedLimb::draw(GUI *gui){
	for (ArticulatedLinkPVector::iterator it = links.begin(), 
             end = links.end(); it != end; ++it) {
        (*it)->draw(gui);

				//draw bounding boxes of the links
				//gui->drawRectangle((*it)->boxMin.x,(*it)->boxMin.y,(*it)->boxMax.x-(*it)->boxMin.x,(*it)->boxMax.y-(*it)->boxMin.y);
    }
}

void ArticulatedLimb::setOutlineColor(Color color){
	for (ArticulatedLinkPVector::iterator it = links.begin(), 
             end = links.end(); it != end; ++it) {
        (*it)->setOutlineColor(color);
    }
}

void ArticulatedLimb::setFillColor(Color color){
	for (ArticulatedLinkPVector::iterator it = links.begin(), 
             end = links.end(); it != end; ++it) {
        (*it)->setFillColor(color);
    }
}