/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// ArticulatedLimb.h: interface for the ArticulatedLimb class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ARTICULATEDLIMB_H
#define ARTICULATEDLIMB_H

#include "ArticulatedLink.h"

class Simulator;

class ArticulatedLimb {
public:
	/** Constructor, sets l and theta. */
	ArticulatedLimb(real l, real theta);

    /** Destructor, deletes the links. */
	virtual ~ArticulatedLimb();
	
	/** Updates the values of sin, cos, sin2, cos2, sinCos as a function of theta*/
	void computeSinCos();

	/** Sets l, theta, calls computeSinCos(). */
	void setProperties(real l, real theta);

	ArticulatedLink* addLink();

	/** Adds a link composed of one object. The link will be articulated around a point situated
	at distance l on the parent link.*/
	ArticulatedLink* addOneObjectLink(real l, real theta0, 
		PhysicalObject* object, real x, real y, real alpha, ArticulatedLink* parent=NULL, 
		bool hasElasticTorque=true);

	virtual void draw(GUI *gui);

	virtual void setOutlineColor(Color color);
	virtual void setFillColor(Color color);

	std::string label;

public:
	real l;
	real theta;

	/** Functions of theta */
	real sinTheta, cosTheta, sin2, cos2, sinCos;

	/** A list containing the links that compose the limb. 
	The list owns the links. */
	ArticulatedLinkPVector links;

	/** Deletes the links list and the member links. */
	void deleteLinks();
};

typedef std::vector<ArticulatedLimb*> ArticulatedLimbPVector;

#endif //ARTICULATEDLIMB_H
