/* 
Thyrix, Copyright (C) 2004-2006 Razvan Florian / Coneural. All rights reserved.
http://www.thyrix.com 

Thyrix Lite is free software. You can redistribute it and/or
modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 
of the License, or (at your option) any later version. The text 
of the GNU General Public License is included with this library 
in the file GeneralPublicLicense.txt.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the file
GeneralPublicLicense.txt for more details.
*/

// MathTools.h: interface for the MathTools class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MATHTOOLS_H__D0539AA7_8BC3_4A5F_99A3_730FAE12C22E__INCLUDED_)
#define AFX_MATHTOOLS_H__D0539AA7_8BC3_4A5F_99A3_730FAE12C22E__INCLUDED_

#include <math.h>

class MathTools  {
public:
	MathTools() {}
	virtual ~MathTools() {}

	/** Defines pi (3.14...) in case that it is not defined by the C library used.*/
	static const double pi;

	/** Defines the size of a degree in radians. */
	static const double degrees;

	template<class Number> inline static Number sqr(Number x){ 
		return x*x;
	}

	template<class Number> inline static int sign(Number x){ 
		return x>0 ? 1 : ( x==0 ? 0 : -1);
	}

	/** Returns a number having the absolute value of x and the sign sign. */
	template<class Number> inline static double setSign(double x, Number sign){
		return sign>=0 ? fabs(x) : -fabs(x);
	}

	template<class Number> inline static double max(Number a, Number b){
		return a>b ? a : b;
	}

	/** Returns sqrt(x^2+y^2) without destructive underflow or overflow. 
	After Numerical Recipes, p. 70 (Press et al.)*/
	inline static double modulus(double x, double y){
		double ax=fabs(x), ay=fabs(y);
		if(ax>ay) return ax*sqrt(1.0+sqr(ay/ax));
		else return (ay==0.0 ? 0.0 : ay*sqrt(1.0+sqr(ax/ay)) );
	}
	

};

#endif // !defined(AFX_MATHTOOLS_H__D0539AA7_8BC3_4A5F_99A3_730FAE12C22E__INCLUDED_)
